<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Fm00020000SolicitacoesFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    // Busca as solicitações
    public function getRequests()
    {
        $sql = "SELECT SOL.NRSOLICITACAO, SOL.NMSOLICITACAO, TO_CHAR(SOL.DTSOLICITACAO, 'DD/MM/YYYY HH24:MI') AS DTSOLICITACAO, SOL.IDSITUACAO,SOL.IDPRIORIDADE, SOL.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE,
            SOL.CDOPERABERTURA, FUNC.NMOPERADOR AS NMFUNCABERTURA, SOL.DSSOLICITACAO, LOCP.NMLOCALCLIENTE AS NMLOCALCLIENTEPAI,
                    (CASE SOL.IDSITUACAO
                        WHEN '1' THEN 'Pendente'
                        WHEN '2' THEN 'Em andamento'
                        WHEN '3' THEN 'Aguardando Avaliação'
                        WHEN '4' THEN 'Concluída'
                        ELSE NULL
                    END) AS IDSITUACAOTEXT,
                    (CASE SOL.IDPRIORIDADE
                        WHEN '1' THEN 'Baixa'
                        WHEN '2' THEN 'Média'
                        WHEN '3' THEN 'Alta'
                        WHEN '4' THEN 'Urgente'
                        ELSE 'A classificar'
                    END) AS IDPRIORIDADETEXT,
                    (CASE  NVL(TSOL.NMTIPOSOLICITACAO, 0)
                        WHEN '0' THEN 'A classificar'
                        ELSE TSOL.NMTIPOSOLICITACAO
                    END) AS NMTIPOSOLICITACAO, TSOL.NRTIPOSOLICITACAO,
                    (CASE  NVL(FUNC2.NMOPERADOR, 0)
                        WHEN '0' THEN 'A classificar'
                        ELSE FUNC2.NMOPERADOR
                    END) AS NMFUNCRESPONSAVEL, SOL.CDOPERRESPONSAVEL
                 FROM FCM_SOLICITACAO SOL
                 JOIN OPERADOR FUNC 
                   ON FUNC.CDOPERADOR = SOL.CDOPERABERTURA
            LEFT JOIN OPERADOR FUNC2 
                   ON FUNC2.CDOPERADOR = SOL.CDOPERRESPONSAVEL
                 JOIN FCM_LOCALCLI LOC 
                   ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
            LEFT JOIN FCM_TIPOSOLICITACAO TSOL 
                   ON TSOL.NRTIPOSOLICITACAO = SOL.NRTIPOSOLICITACAO
                 JOIN FCM_LOCALCLI LOCP 
                   ON LOCP.NRLOCALCLIENTE = LOC.NRLOCALCLIENTEPAI
             ORDER BY SOL.DTSOLICITACAO ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Busca os tipos de solicitação
    public function getRequestTypes()
    {
        $sql = "SELECT NRTIPOSOLICITACAO, NMTIPOSOLICITACAO, 
                        (CASE IDPRIORIDADEPADR
                            WHEN '1' THEN 'Baixa'
                            WHEN '2' THEN 'Média'
                            WHEN '3' THEN 'Alta'
                            WHEN '4' THEN 'Urgente'
                            ELSE NULL
                        END) AS IDPRIORIDADEPADR
                    FROM FCM_TIPOSOLICITACAO";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }


    // Busca os tipos de solicitação para o grid
    public function getTipoSolicitacao()
    {
        $sql = "SELECT NRTIPOSOLICITACAO, NMTIPOSOLICITACAO, IDATIVO,
                        (CASE IDPRIORIDADEPADR
                            WHEN '1' THEN 'Baixa'
                            WHEN '2' THEN 'Média'
                            WHEN '3' THEN 'Alta'
                            WHEN '4' THEN 'Urgente'
                            ELSE NULL
                        END) AS IDPRIORIDADEPADR
                    FROM FCM_TIPOSOLICITACAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao)
    {
        $sql = "SELECT SOL.NRSOLICITACAO, SOL.NMSOLICITACAO, TO_CHAR(SOL.DTSOLICITACAO, 'DD/MM/YYYY HH24:MI') AS DTSOLICITACAO, SOL.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE, SOL.CDOPERABERTURA, FUNC.NMOPERADOR AS NMFUNCABERTURA, SOL.DSSOLICITACAO,
                    (CASE SOL.IDSITUACAO
                        WHEN '1' THEN 'Pendente'
                        WHEN '2' THEN 'Em andamento'
                        WHEN '3' THEN 'Aguardando Avaliação'
                        WHEN '4' THEN 'Concluída'
                        ELSE NULL
                    END) AS IDSITUACAO,
                    (CASE SOL.IDPRIORIDADE
                        WHEN '1' THEN 'Baixa'
                        WHEN '2' THEN 'Média'
                        WHEN '3' THEN 'Alta'
                        WHEN '4' THEN 'Urgente'
                        ELSE 'A classificar'
                    END) AS IDPRIORIDADE,
                    (CASE  NVL(TSOL.NMTIPOSOLICITACAO, 0)
                        WHEN '0' THEN 'A classificar'
                        ELSE TSOL.NMTIPOSOLICITACAO
                    END) AS NMTIPOSOLICITACAO, TSOL.NRTIPOSOLICITACAO,
                    (CASE  NVL(FUNC2.NMOPERADOR, 0)
                        WHEN '0' THEN 'A classificar'
                        ELSE FUNC2.NMOPERADOR
                    END) AS NMFUNCRESPONSAVEL, SOL.CDOPERRESPONSAVEL
                FROM FCM_SOLICITACAO SOL
                JOIN OPERADOR FUNC 
                ON FUNC.CDOPERADOR = SOL.CDOPERABERTURA
            LEFT JOIN OPERADOR FUNC2 
                ON FUNC2.CDOPERADOR = SOL.CDOPERRESPONSAVEL
                JOIN FCM_LOCALCLI LOC 
                ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
            LEFT JOIN FCM_TIPOSOLICITACAO TSOL 
                ON TSOL.NRTIPOSOLICITACAO = SOL.NRTIPOSOLICITACAO
            WHERE SOL.NRSOLICITACAO = {$nrsolicitacao}";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // Busca o histórico da solicitação
    public function getRequestHistoric($parametros)
    {
        /* WHEN '1' THEN 'Solicitação Criada'
                                    WHEN '2' THEN 'Solicitação Classificada'
                                    WHEN '3' THEN 'Aguardando Avaliação'
                                    WHEN '4' THEN 'Solicitação Avaliada' */
                    
        $sql = "SELECT HIST.CDOPERSOLICITANTE, HIST.NMSOLICITACAO, HIST.NRSOLICITACAOHISTORICO, FUNC.NMOPERADOR, TO_CHAR(HIST.DTHISTSOLICITACAO, 'DD/MM/YYYY HH24:MI') AS DTHISTSOLICITACAO, 
                LOC.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE, HIST.DSSOLICITACAO,  HIST.DTINCLUSAO, TO_CHAR(HIST.DTULTATU, 'DD/MM/YYYY HH24:MI') AS DTULTATU, HIST.CDOPERULTATU, OP.NMOPERADOR AS NMOPERADORULTATU,
                HIST.IDTIPOHISTORICO
                        FROM FCM_SOLICITACAOHISTORICO HIST
                        JOIN OPERADOR FUNC 
                          ON FUNC.CDOPERADOR = HIST.CDOPERSOLICITANTE
                        JOIN FCM_SOLICITACAO SOL 
                          ON SOL.NRSOLICITACAO = HIST.NRSOLICITACAO
                        JOIN FCM_LOCALCLI LOC 
                          ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
                        LEFT JOIN OPERADOR OP 
                          ON OP.CDOPERADOR = HIST.CDOPERULTATU
                         WHERE HIST.NRSOLICITACAO = :NRSOLICITACAO
                        ORDER BY HIST.DTHISTSOLICITACAO DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }


    public function insertRequestType($typeParams)
    {

        $sql = "INSERT INTO FCM_TIPOSOLICITACAO (NRTIPOSOLICITACAO, NMTIPOSOLICITACAO, IDPRIORIDADEPADR, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, IDATIVO)
                    VALUES (:NRTIPOSOLICITACAO, :NMTIPOSOLICITACAO, :IDPRIORIDADEPADR, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, 'S')";

        $this->connection->executeStatement($sql, $typeParams);
    }

    // Atualiza a tabela de solicitação
    public function updateRequest($requestParams)
    {
        $sql = "UPDATE FCM_SOLICITACAO
                    SET NMSOLICITACAO = :NMSOLICITACAO,
                        IDSITUACAO = :IDSITUACAO,
                        NRTIPOSOLICITACAO = :NRTIPOSOLICITACAO,
                        IDPRIORIDADE = :IDPRIORIDADE,
                        CDOPERRESPONSAVEL = :CDOPERRESPONSAVEL,
                        DSSOLICITACAO = :DSSOLICITACAO,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU
                 WHERE NRSOLICITACAO = :NRSOLICITACAO";

        $this->connection->executeStatement($sql, $requestParams);
    }

    // Cria uma tarefa
    public function createTask($checklistParams)
    {
        $sql = "INSERT INTO FCM_CHECKLISTAGEND (NRCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST,                                                                             NRFUNCIONARIO, DTHORAAGEND, NRORG, DTINCLUSAO, 
                                                NMCHECKLISTAGEND, DTHRFINALAGEND)
                    VALUES (:NRCHECKLISTAGEND, :NRLOCALCLIENTE, :NRCHECKLIST, :NRFUNCIONARIO,
                            SYSDATE, :NRORG, SYSDATE, :NMCHECKLISTAGEND, SYSDATE+0.01)";
        $this->connection->executeStatement($sql, $checklistParams);
    }

    // Cria uma relação da tarefa com solicitação
    public function createTaskRequest($taskRequestParams)
    {
        $sql = "INSERT INTO FCM_SOLICITACAOTAREFAS (NRSOLICITACAOTAREFAS, NRSOLICITACAO, NRCHECKLISTAGEND, NRORG, DTINCLUSAO, DTULTATU, CDOPERULTATU)
                    VALUES (:NRSOLICITACAOTAREFAS, :NRSOLICITACAO, :NRCHECKLISTAGEND, :NRORG, SYSDATE, SYSDATE, :CDOPERULTATU)";
                
        $this->connection->executeStatement($sql, $taskRequestParams);
    }

    // insere um dado no histórico
    public function insertHistoricData($parametros)
    {
        $sql = "INSERT INTO FCM_SOLICITACAOHISTORICO (NRSOLICITACAOHISTORICO, NRSOLICITACAO, NMSOLICITACAO, DTHISTSOLICITACAO,
                        IDSITUACAO, IDTIPOHISTORICO, CDOPERSOLICITANTE, DSSOLICITACAO, NRORG, DTINCLUSAO, DTULTATU, CDOPERULTATU)
                VALUES (:NRSOLICITACAOHISTORICO, :NRSOLICITACAO, :NMSOLICITACAO, SYSDATE,
                            :IDSITUACAO, :IDTIPOHISTORICO, :CDOPERSOLICITANTE, :DSSOLICITACAO, :NRORG, SYSDATE, SYSDATE, :CDOPERULTATU)";
                        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateRequestType($typeParams)
    {
        $sql = " UPDATE FCM_TIPOSOLICITACAO
                    SET NMTIPOSOLICITACAO = :NMTIPOSOLICITACAO,
                        IDPRIORIDADEPADR = :IDPRIORIDADEPADR,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU,
                        IDATIVO = :IDATIVO
                WHERE NRTIPOSOLICITACAO = :NRTIPOSOLICITACAO";

        $this->connection->executeStatement($sql, $typeParams);
    }

    public function inativarTipoSolicitacao($typeParams)
    {
        $sql = "UPDATE FCM_TIPOSOLICITACAO
                   SET IDATIVO      = :IDATIVO,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU  = :NRORGULTATU
                 WHERE  NRTIPOSOLICITACAO = :NRTIPOSOLICITACAO";

        $this->connection->executeStatement($sql, $typeParams);
    }

    public function getLocalRequest($params)
    {
        $sqlCode = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE,
                           FCM_LOCALCLI.NRSETOR, FCM_LOCALCLI.NRLOCALCLIENTEPAI
                     FROM FCM_LOCALCLI
                     JOIN FCM_TIPOLOCALCLI
                       ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = " . $params["parent"];

        return $this->connection->executeQuery($sqlCode)->fetchAllAssociative();
    }

    public function insertNewRequest($params)
    {

        $sql = "INSERT INTO FCM_SOLICITACAO (NRSOLICITACAO, NMSOLICITACAO, DTSOLICITACAO, IDSITUACAO, NRLOCALCLIENTE,
            NRFUNCABERTURA, DSSOLICITACAO, NRORG, IDATIVO, DTINCLUSAO)
              VALUES (:NRSOLICITACAO, :NMSOLICITACAO, SYSDATE, :IDSITUACAO, :NRLOCALCLIENTE, :NRFUNCABERTURA,
                      :DSSOLICITACAO, :NRORG, 'S', SYSDATE)";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertRequestHistory($params)
    {

        $sql = "INSERT INTO FCM_SOLICITACAOHISTORICO (NRSOLICITACAOHISTORICO, NRSOLICITACAO, DTHISTSOLICITACAO,
            IDSITUACAO, IDTIPOHISTORICO, NRFUNCSOLICITANTE, DSSOLICITACAO, IDSATISFACAOSOLICITA, NRORG, IDATIVO, CDOPERULTATU)
              VALUES (:NRSOLICITACAOHISTORICO, :NRSOLICITACAO, SYSDATE, :IDSITUACAO, :IDTIPOHISTORICO,
              :NRFUNCSOLICITANTE, :DSSOLICITACAO, :IDSATISFACAOSOLICITA, :NRORG, 'S', :CDOPERULTATU)";

        $this->connection->executeStatement($sql, $params);
    }

    public function getSituationRequest()
    {

        $sql = "SELECT DISTINCT S.IDSITUACAO AS NRIDSITUACAO,
                  CASE S.IDSITUACAO WHEN '1' THEN 'Pendente'
                     WHEN '2' THEN 'Em andamento'
                     WHEN '3' THEN 'Aguardando Avaliação'
                     WHEN '4' THEN 'Concluído'
                     ELSE NULL END AS IDSITUACAO
                     FROM FCM_SOLICITACAO S";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getNameRequest()
    {

        $sql = "SELECT S.NMSOLICITACAO, S.NRSOLICITACAO
                  FROM FCM_SOLICITACAO S";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getCardsRequests($parametros, $usersIds)
    {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO, TO_CHAR(S.DTSOLICITACAO, 'DD/MM HH24:MI') DTSOLICITACAO, S.NRLOCALCLIENTE, S.IDSITUACAO, LC.NMLOCALCLIENTE,
                  CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                 FROM FCM_SOLICITACAO S
                 JOIN FCM_LOCALCLI LC
                   ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                   WHERE S.NRFUNCABERTURA IN ({$usersIds})" . "$parametros";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function showRequest($param)
    {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO, S.DSSOLICITACAO, TO_CHAR(S.DTSOLICITACAO, 'DD/MM HH24:MI') DTSOLICITACAO, S.NRLOCALCLIENTE, S.IDSITUACAO, LC.NMLOCALCLIENTE,
                    CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                FROM FCM_SOLICITACAO S
                JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                WHERE S.NRSOLICITACAO = :NRSOLICITACAO";

        return $this->connection->executeQuery($sql, $param)->fetchAllAssociative();
    }

    public function getTarefaChk($param)
    {

        $sql = "SELECT NRTAREFACHK FROM FCM_TAREFACHK WHERE NRCHECKLIST = :NRCHECKLIST";

        return $this->connection->executeQuery($sql, $param)->fetchAllAssociative();
    }

    public function upRequest($parametros)
    {

        $sql = "UPDATE FCM_SOLICITACAO
                  SET IDSITUACAO = :IDSITUACAO,
                      DSSOLICITACAO = :DSSOLICITACAO
                WHERE NRSOLICITACAO = :NRSOLICITACAO";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function newTarefa($parametro)
    {
        $sql = "INSERT INTO FCM_TAREFA (NRTAREFA, NMTAREFA, NRATIVIDADE, IDATIVO, DTINCLUSAO)
            VALUES (:NRTAREFA, :NMTAREFA, :NRATIVIDADE,'S', SYSDATE)";

        $this->connection->executeStatement($sql, $parametro);
    }

    public function newTarefaChk($parametro)
    {
        $sql = "INSERT INTO FCM_TAREFACHK (NRTAREFACHK, NRTAREFA, NRCHECKLIST, NRATIVIDADECHK, IDATIVO, DTINCLUSAO)
            VALUES (:NRTAREFACHK, :NRTAREFA, :NRCHECKLIST, :NRATIVIDADECHK,'S', SYSDATE)";

        $this->connection->executeStatement($sql, $parametro);
    }

    public function insertNovaTarefa($param)
    {
        $sql = "INSERT INTO FCM_CHECKLISTAGEND (NRCHECKLISTAGEND, NMCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST, CDOPERADOR, 
                IDSITUACAOAGEND, DTHORAAGEND,  IDATIVO, DTINCLUSAO, DTHRFINALAGEND, CDOPERINCLUSAO, DTULTATU, CDOPERULTATU, NRORG)
                VALUES (:NRCHECKLISTAGEND, :NMCHECKLISTAGEND, :NRLOCALCLIENTE, :NRCHECKLIST, :CDOPERADOR, :IDSITUACAOAGEND, 
                TO_DATE(:DTHORAAGEND,'YYYY/MM/DD HH24:MI'),  'S', SYSDATE, TO_DATE(:DTHRFINALAGEND,'YYYY/MM/DD HH24:MI'), :CDOPERINCLUSAO, SYSDATE, :CDOPERULTATU, :NRORG)";

        $this->connection->executeStatement($sql, $param);
    }

    public function insertNovaTarefa2($param)
    {
        $sql = "INSERT INTO FCM_TAREFAAGEND (NRTAREFAAGEND, NRCHECKLISTAGEND, NRTAREFACHK, CDOPERADOR, DTHORATAREFAAGEND, DSOBSTAREFAAGEND, IDREALIZADO, 
                IDATIVO, DTINCLUSAO, CDOPERINCLUSAO, DTULTATU)
                VALUES (:NRTAREFAAGEND, :NRCHECKLISTAGEND, :NRTAREFACHK, :CDOPERADOR, TO_DATE(:DTHORAAGEND,'YYYY/MM/DD HH24:MI'), 
                :DSOBSTAREFAAGEND, 'N',  'S', SYSDATE, :CDOPERINCLUSAO, SYSDATE)";
    
        $this->connection->executeStatement($sql, $param);
    }
}
